from argparse import ArgumentParser
from typing import Dict, Union
import loguru
import sys

loguru.logger.remove()
loguru.logger.add(sys.stdout, level="WARNING")

import vkbottle
from vkbottle.bot import Message
import threading
import asyncio
from functools import partial
from dataclasses import dataclass
from random import randint


@dataclass
class MetaArgs:
    loop: asyncio.AbstractEventLoop
    user_id: int
    peer_id: int


parser = ArgumentParser("intobot", description="This converts your script to a bot.")

parser.add_argument("script")
parser.add_argument("-t", "--token", required=True)

args = parser.parse_args()

token = open(args.token).read().strip()

script = open(args.script, encoding="utf-8").read()

bot = vkbottle.Bot(token=token)

user_ids_to_states: Dict[int, Union[str, bool]] = {}


async def async_fake_print(meta, message):
    message = str(message)
    await bot.api.messages.send(
        peer_id=meta.peer_id,
        message=message,
        random_id=randint(-1_000_000, 1_000_000),
    )


def fake_print(meta, message):
    asyncio.run_coroutine_threadsafe(async_fake_print(meta, message), meta.loop)


def fake_input(meta, query=""):
    user_ids_to_states[meta.user_id] = True
    if str(query):
        fake_print(meta, query)
    while type(user_ids_to_states[meta.user_id]) is not str:
        pass
    response = user_ids_to_states[meta.user_id]
    user_ids_to_states[meta.user_id] = False
    return response


def run(meta):
    exec(
        script,
        {
            "print": partial(fake_print, meta),
            "input": partial(fake_input, meta),
        },
        {},
    )


@bot.on.message()
async def on_message(message: Message):
    user_id = message.from_id
    loop = asyncio.get_running_loop()
    try:
        state = user_ids_to_states[user_id]
    except KeyError:
        user_ids_to_states[user_id] = False
        threading.Thread(
            target=run,
            args=(MetaArgs(
                peer_id=message.peer_id,
                user_id=message.from_id,
                loop=loop
            ),),
        ).start()
    else:
        if state is True:
            user_ids_to_states[user_id] = message.text


print("Starting!")
bot.run_forever()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['intobot']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.6.0,<0.7.0', 'vkbottle>=4.3.12,<5.0.0']

setup_kwargs = {
    'name': 'intobot',
    'version': '0.1.1',
    'description': 'A tool to make every script into a VK bot. Better than `every_script_is_a_vk_bot`, because `intobot` can serve multiple users simultaneously.',
    'long_description': '# intobot\n\nThis tool can turn your script into a VK bot. In every script launched with this tool, `input` and `print` will work with chats instead of the console. A script is ran one time for every user who writes to the bot. First message from a user starts the script execution and cannot be received from the script.\n\nInstallation:\n\n    pip install intobot\n\nUse it like this:\n\n    python -m intobot -t [token_file_name.txt] [script_name].py\n\nEverything above that is in brackets needs to be filled by you. (Brackets should be omitted.)\n\nWhen the script ends, the user is not served anymore, so to serve the user infinitely, add an infinite loop to the script. This was made to allow better flexibility and make it as close to console execution of scripts as possible.\n\n[Predecessor: every_script_is_a_vk_bot](https://github.com/megahomyak/every_script_is_a_vk_bot)\n',
    'author': 'megahomyak',
    'author_email': 'g.megahomyak@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
